; This program is free software. It comes without any warranty, to
; the extent permitted by applicable law. You can redistribute it
; and/or modify it under the terms of the Do What The Fuck You Want
; To Public License, Version 2, as published by Sam Hocevar. See
; http://sam.zoy.org/wtfpl/COPYING for more details.

; This file contains offsets of memory locations and constants that replace
; ti83plus.inc or ti84pcse.inc .

#define equ	.equ

objTblOffset	.equ	12

;------ Macros -----------------------------------------------------------------

#macro getLcdRegHl(regnum)
	ld a, regnum
	out (pLcdCmd), a
	out (pLcdCmd), a
	in a, (pLcdData)
	ld h, a
	in a, (pLcdData)
	ld l, a
#endmacro

#macro setLcdRegHl(regnum)
	ld a, regnum
	out (pLcdCmd), a
	out (pLcdCmd), a
	ld a, h
	out (pLcdData), a
	ld a, l
	out (pLcdData), a
#endmacro

#macro setLcdReg(regnum, highbyte, lowbyte)
	ld a, regnum
	out (pLcdCmd), a
	out (pLcdCmd), a
	ld a, highbyte
	out (pLcdData), a
	ld a, lowbyte
	out (pLcdData), a
#endmacro

#macro setLcdDReg(regnum, double)
	ld a, regnum
	out (pLcdCmd), a
	out (pLcdCmd), a
	ld a, lcdHigh(double)
	out (pLcdData), a
	ld a, lcdLow(double)
	out (pLcdData), a
#endmacro

#macro cpHlDe
	or	a
	sbc	hl, de
	add	hl, de
#endmacro


;------ Structs ----------------------------------------------------------------
; 32-bit page/address pointer


;------ Main Memory Layout Settings --------------------------------------------
;plotSScreen	equ	987Ch
;saveSScreen	equ	8798h
;textShadow	equ	08560h	; 260 bytes
;cmdShadow	equ	09BAAh ; Also expanded
;statVars		equ	8C1Ch	; scrap 531 (clear b_call(_DelRes))
;appData	equ	8000h

;objectArray	.equ	saveSScreen
;itemCache	.equ	objectArray + 256
objectArray	.equ	appData
itemCache	.equ	saveSScreen
itemCacheSize	.equ	766
huffmanTable	.equ	cmdShadow
start_of_static_vars	.equ	textShadow
; 9A01 is not free! It's the last byte of the IVT.
stringFindFreeRAM .equ	987Ch ; formerly plotSScreen
IvtLocation	.equ	99h ; vector table at 9900h
IsrLocation	.equ	98h ; ISR at 9898h
; Memory mapping
swapPort		.equ	6
swapPortHigh		.equ	0Eh
swapLoc			.equ	4000h
swapTop			.equ	8000h


;------ Character Constants ----------------------------------------------------
chQuotes	.equ	22h
chNewLine	.equ	01h
ch1stPrintableChar	.equ	1Fh
chThickSpace	.equ	7Fh


;------ Screen & Text ----------------------------------------------------------
colorScrnHeight = 240
colorScrnWidth = 320
minBacklightLevel = 32
defaultBrightness = 10h
textCols = 32
textRows = 17
charHeight = 14
charWidth = 10


;------ Settings ---------------------------------------------------------------
; Timers
suspendDelay	.equ	180	; 3 minutes
kbdScanDivisor		.equ	10	; The below values are multiples of 10.24 ms
minKeyHoldTime		.equ	3	; 
keyBlockTime		.equ	1	; 
keyFirstRepeatWaitTime	.equ	25	; 
keyRepeatWaitTime	.equ	05	; 

shortWait	.equ	100;500
longWait	.equ	2000;6000
mediumWait	.equ	1000;3000


;------ Flags ------------------------------------------------------------------
; Flags


;------ Vars -------------------------------------------------------------------

;start_of_static_vars .equ saveSScreen
; General Variables
; start_of_static_vars is defined in the equates_bw or equates_pcse

; Game stuff
spInitial	.equ	start_of_static_vars
pageAInitial	.equ	spInitial + 2
pageAHighInitial	.equ	pageAInitial + 1
cursorY	.equ	pageAHighInitial + 1
cursorX	.equ	cursorY + 1
randomVal	.equ	cursorX + 1
seed1	.equ	randomVal
seed2	.equ	randomVal + 2
dataFilePage	.equ	seed2 + 2;randomVal + 2
dataFileLoc	.equ	dataFilePage + 1
rotationTimer	.equ	dataFileLoc + 2
itemCount	.equ	rotationTimer + 1
objectCount	.equ	itemCount + 2
currentByte	.equ	objectCount + 1
currentBit	.equ	currentByte + 1
currentReadLoc	.equ	currentBit + 1
currentReadPage	.equ	currentReadLoc + 2
foundObject	.equ	currentReadPage + 1
stringOffset	.equ	foundObject + 1
stringStage	.equ	stringOffset + 2
scrollTimer	.equ	stringStage + 1
lastItemPtr	.equ	scrollTimer + 2
fpItems		.equ	lastItemPtr + 2
OP7		.equ	fpItems + 9
end_of_game_vars	.equ	OP7 + 9

; APD
genFastTimer	.equ	end_of_game_vars
suspendTimer	.equ	genFastTimer + 2
end_of_apd_vars	.equ	suspendTimer + 2

; Keyboard
kbdScanTimer		.equ	end_of_apd_vars
lastKey			.equ	kbdScanTimer + 1	; Normally 0
lastKeyHoldTimer	.equ	lastKey + 1		; Normally minKeyHoldTime
lastKeyBlockTimer	.equ	lastKeyHoldTimer + 1	; Normally keyBlockTime
lastKeyFirstRepeatTimer	.equ	lastKeyBlockTimer + 1	; Normally keyFirstRepeatWaitTime
lastKeyRepeatTimer	.equ	lastKeyFirstRepeatTimer + 1	; Normally keyRepeatWaitTime
keyBuffer		.equ	lastKeyRepeatTimer + 1	; Normally 0
end_of_kbd_vars	.equ	keyBuffer + 1

; Screen
currentRow	.equ	end_of_kbd_vars
currentCol	.equ	currentRow + 1
textColors	.equ	currentCol + 1
textForeColor	.equ	textColors
textBackColor	.equ	textForeColor + 1
lcdCol		.equ	textBackColor + 1

