; This program is free software. It comes without any warranty, to
; the extent permitted by applicable law. You can redistribute it
; and/or modify it under the terms of the Do What The Fuck You Want
; To Public License, Version 2, as published by Sam Hocevar. See
; http://sam.zoy.org/wtfpl/COPYING for more details.


; struct objectEntry
; {
; 	byte y;
; 	byte x;
; 	short itemId;
;	char ascii;
;	byte color;
; }

;------ StartGame --------------------------------------------------------------
StartGame:
	; Reset some vars
	ld	hl, 0
	ld	(foundObject), hl
	ld	(stringOffset), hl
	ld	(stringStage), hl
	ld	(scrollTimer), hl
	; Clear object table
	ld	a, 255
	ld	hl, objectArray
	call	ClearMem
	; Erase intro text
	call	ClrScrnFull
	; First, generate objects set
	ld	ix, objectArray
	call	Random16
	and	7
	add	a, 20
	ld	(objectCount), a
	ld	iy, stringStage
	ld	(iy), a
generateLoop:
	; Random location
	call	Random16
	ld	a, l
	and	0Fh
	ld	e, a
	ld	a, h
	and	1Fh
	ld	d, a
	call	TestCollision
	jr	c, generateLoop
	ld	(ix), e
	ld	(ix+1), d
	; Random item
	call	Random16
	ld	c, l
	ld	a, h
	ld	de, (itemCount)
	call	DivACByDE
	ld	(ix+2), l
	ld	(ix+3), h
	; Random char and color
rchlp:	call	Random16
	; Verify that char is printable
	ld	a, l
	and	7Fh
	cp	'!'
	jr	c, rchlp
	cp	7Fh
	jr	z, rchlp
	cp	'#'
	jr	z, rchlp
	ld	(ix+4), a;l
	; Verify that color is not too dark
	; These values found empirically
	ld	a, h
	ld	hl, colorsBlacklist
	ld	b, (hl)
	inc	hl
_:	cp	(hl)
	jr	z, rchlp
	djnz	-_
	; OK!
	ld	(ix+5), a; 77h ; should be h register
	ld	l, (ix)
	ld	h, (ix+1)
	call	Locate
	ld	a, (ix+5)
	ld	(textForeColor), a
	ld	a, (ix+4)
	call	PutC
	ld	de, 6
	add	ix, de
	dec	(iy)
	jr	nz, generateLoop
	ld	a, 255
	ld	(textForeColor), a
; Random location for kitten
; Just use the last generated random item
	ld	(ix-3), 255
	ld	(ix-4), 255
	
;	ld	l, (ix-6)
;	ld	h, (ix-5)
;	call	Locate
;	ld	a, (ix-1)
;	ld	(textForeColor), a
;	ld	a, '!'
;	call	PutC
;	call	PutC
;	call	PutC
;	ld	a, 255
;	ld	(textForeColor), a
	
; Initial Location
_:	call	Random16
	ld	a, l
	and	0Fh
	ld	e, a
	ld	a, h
	and	1Fh
	ld	d, a
	call	TestCollision
	jr	c, -_
	ld	(cursorY), de


;------ GameLoop ---------------------------------------------------------------
GameLoop:
	ld	hl, (cursorY)
	call	Locate
	ld	a, '#'
	call	PutC
	
getKeyLoop:
	ld	a, (foundObject)
	or	a
	jp	z, getKeyLoopKeyGet
	ld	hl, (scrollTimer)
	dec	hl
	ld	(scrollTimer), hl
	ld	a, (stringStage)
	cp	1
	jp	z, stringPause1
	cp	2
	jr	z, stringScroll
	cp	3
	jp	z, stringPause2
stringRestart:
; This string scrolling code is recycled largely unchanged from my old B&W RFK.
	ld	a, 1
	ld	(stringStage), a
	xor	a
	ld	(foundObject), a
	ld	hl, 16
	call	Locate
	ld	hl, itemCache
	call	PutS
	push	af
	call	ClearEOL
	pop	af
	jp	c, getKeyLoopKeyGet
	ld	hl, 1F10h
	call	Locate
	ld	a, '.'
	call	PutC
	call	PutC
	call	PutC
	ld	a, 1
	ld	(foundObject), a
	ld	hl, mediumWait
	ld	(scrollTimer), hl
	ld	hl, itemCache
	ld	(stringOffset), hl
	jp	getKeyLoopKeyGet
stringScroll:
	ld	hl, (scrollTimer)
	ld	de, 0FFFFh
	add	hl, de	; flags
	jr	c, getKeyLoopKeyGet
	ld	hl, 16
	call	Locate
	ld	hl, (stringOffset)
	inc	hl
	ld	(stringOffset), hl
	call	PutS
	push	af
	call	ClearEOL
	pop	af
	jr	c, stringScrollDispStringDoneScroll
	ld	hl, 1F10h
	call	Locate
	ld	a, '.'
	call	PutC
	call	PutC
	call	PutC
	ld	hl, shortWait
	ld	(scrollTimer), hl
	jr	getKeyLoopKeyGet
stringScrollDispStringDoneScroll:
	ld	hl, longWait
	ld	(scrollTimer), hl
	ld	a, (stringStage)
	inc	a
	ld	(stringStage), a
	jr	getKeyLoopKeyGet
stringPause1:
stringPause2:
	ld	hl, (scrollTimer)
	ld	de, 0FFFFh
	add	hl, de
	jr	c, getKeyLoopKeyGet
	ld	hl, shortWait
	ld	(scrollTimer), hl
	ld	a, (stringStage)
	inc	a
	ld	(stringStage), a
	
getKeyLoopKeyGet:
	ei
	halt
	call	GetCSC
	or	a
	jp	z, getKeyLoop

	push	af
	ld	hl, (cursorY)
	call	Locate
	ld	a, chThickSpace
	call	PutC
	pop	af
	cp	skUp
	jr	z, glUp
	cp	skDown
	jr	z, glDown
	cp	skLeft
	jr	z, glLeft
	cp	skRight
	jr	z, glRight
	cp	skClear
	jp	z, Quit
;	cp	skGraph
;	jp	z, StartGame
;	cp	skGraph
;	jp	z, redraw
	jp	GameLoop
glUp:
	ld	de, (cursorY)
	ld	a, e
	or	a
	jp	z, GameLoop
	dec	a
	ld	e, a
	call	TestCollision
	jr	c, glSetNki
	ld	(cursorY), de
	jp	GameLoop
glDown:
	ld	de, (cursorY)
	ld	a, e
	inc	a
	cp	16
	jp	z, GameLoop
	ld	e, a
	call	TestCollision
	jr	c, glSetNki
	ld	(cursorY), de
	jp	GameLoop
glLeft:
	ld	de, (cursorY)
	ld	a, d
	or	a
	jp	z, GameLoop
	dec	a
	ld	d, a
	call	TestCollision
	jr	c, glSetNki
	ld	(cursorY), de
	jp	GameLoop
glRight:
	ld	de, (cursorY)
	ld	a, d
	inc	a
	cp	32
	jp	z, GameLoop
	ld	d, a
	call	TestCollision
	jr	c, glSetNki
	ld	(cursorY), de
	jp	GameLoop
glSetNki:
	push	hl
	pop	ix
	xor	a
	ld	(stringStage), a
	inc	a
	ld	(foundObject), a
	ld	a, (hl)
	inc	hl
	ld	h, (hl)
	ld	l, a
	cp	255
	jr	nz, +_
	ld	a, h
	cp	255
	jr	z, robotfoundkitten
_:	call	GetMessage
	;ld	hl, 16
	;call	Locate
	;ld	hl, itemCache
	;call	PutS
	;call	ClearEOL
	jp	GameLoop

#ifdef NEVER
redraw:
	call	ClrScrnFull
	ld	ix, objectArray
	ld	a, (objectCount)
	ld	b, a
_:	ld	l, (ix)
	inc	ix
	ld	h, (ix)
	inc	ix
	call	Locate
	inc	ix
	inc	ix
	ld	a, (ix+1)
	inc	ix
	ld	(textForeColor), a
	ld	a, (ix-1)
	inc	ix
	call	PutC
	djnz	-_
	
	ld	a, 255
	ld	(textForeColor), a
	jp	GameLoop
#endif


;------ Robot found kitten! ----------------------------------------------------
robotfoundkitten:
	ld	hl, 0FFh
	ld	(textColors), hl
	call	ClrScrnFull
	
	ld	a, 0FFh
	ld	(textForeColor), a
	ld	hl, endSeq1
	ld	b, 5
	call	PutSCentered
	ld	a, (ix+3)
	ld	(textForeColor), a
	ld	a, (ix+2)
	call	PutC
	call	OneSecondWait
	
	ld	hl, 5
	call	Locate
	call	ClearEOL
	ld	a, 0FFh
	ld	(textForeColor), a
	ld	hl, endSeq2
	ld	b, 5
	call	PutSCentered
	ld	a, (ix+3)
	ld	(textForeColor), a
	ld	a, (ix+2)
	call	PutC
	call	OneSecondWait
	
	ld	hl, 5
	call	Locate
	call	ClearEOL
	ld	a, 0FFh
	ld	(textForeColor), a
	ld	hl, endSeq3
	ld	b, 5
	call	PutSCentered
	ld	a, (ix+3)
	ld	(textForeColor), a
	ld	a, (ix+2)
	call	PutC
	call	OneSecondWait
	
	ld	hl, 5
	call	Locate
	call	ClearEOL
	ld	a, 0FFh
	ld	(textForeColor), a
	ld	hl, endSeq4
	ld	b, 5
	call	PutSCentered
	ld	a, (ix+3)
	ld	(textForeColor), a
	ld	a, (ix+2)
	call	PutC
	call	OneSecondWait
	
	ld	hl, 5
	call	Locate
	call	ClearEOL
	ld	a, 0FFh
	ld	(textForeColor), a
	ld	hl, robotfindskittenMsg
	ld	b, 5
	call	PutSCentered
	call	OneSecondWait

	call	GetCSC
	xor	a
	ld	(keyBuffer), a
	
	ld	hl, 0FFh
	ld	(textColors), hl
	ld	hl, goodJobMsg
	ld	b, 11
	call	PutSCentered

	call	GetKey
	jp	Quit


;====== Routines ===============================================================
;------ TestCollision ----------------------------------------------------------
TestCollision:
; Tests if is an item at the given location.
; Input:
;  - D: X
;  - E: Y
; Output:
;  - C on collision
;  - NC if no collision
;  - HL points to object entry's itemId
;  - HL points to garbage otherwise
; Destroys:
;  - AF
;  - BC
	push	bc
	ld	hl, objectArray
	ld	a, (objectCount)
	ld	b, a
testCollisionLoop:
	ld	a, (hl)
	inc	hl
	cp	e
	ld	a, (hl)
	inc	hl
	jr	nz, +_
	cp	d
	jr	nz, +_
	scf
	pop	bc
	ret
_:	inc	hl
	inc	hl
	inc	hl
	inc	hl
	djnz	testCollisionLoop
	or	a
	pop	bc
	ret


;------ GetRandomColor ---------------------------------------------------------
;GetRandomColor:
	

;------ OneSecondWait ----------------------------------------------------------
OneSecondWait:
; Does what it says.
; Destroys:
;  - HL
;  - AF
	ld	hl, 0
	ld	(genFastTimer), hl
_:	ei
	halt
	ld	hl, (genFastTimer)
	ld	a, l
	or	h
	jr	nz, -_
	ret
